local AddonName, Addon = ...
local L = Addon:GetLocale()
local UI = Addon.CommonUI.UI
local Encoder = Addon.Features.Import.Encoder
local ImportDialog = {}

--[===[@debug@


end
--@end-debug@]===]

local ImportHandlers = {
    customlist = "Features.Import.ImportList",
    customrule = "Features.Import.ImportRule"
}

function ImportDialog:OnInitDialog(dialog, importString)

    self:SetButtonState({ import = false, cancel = true })
end

function ImportDialog:OnShow()
end

function ImportDialog:OnHide()
end

function ImportDialog:OnImportText(text)


    local import = Encoder.Decode(text)
    if (type(import) == "table") then



        
        local handler = ImportHandlers[import.Content]

        if (not handler) then

            -- error 1
            return
        end

        handler = UI.Resolve(handler)
        if (not handler) then

            -- error 2 
            return
        end

        if (not handler:Validate(import)) then

            -- error 4
            return
        else

        end

        local frame = handler:CreateUI(self, import)
        self.handler = handler
        self.payload = import

        self.import:Hide()
        self.help:Hide()
        frame:SetPoint("TOPLEFT", self, "TOPLEFT")
        frame:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT")
        frame:Show()

        local buttons = {}
        buttons.cancel = true
        buttons.confirm = true

        self:SetButtonState(buttons)
    end
end

--[[ Handle importing the actual data ]]
function ImportDialog:DoImport()
    if (self.handler) then
        self.handler:Import(self.payload)
        self:Close()
    end
end

--[[ Show the export dialog with the contents provided ]]
function Addon.Features.Import:ShowImportDialog(importString)
    local dialog = UI.Dialog("IMPORT_DIALOG_CAPTION", "Import_ImportDialog", ImportDialog, {
            cancel = { label = L["EXPORT_CLOSE_BUTTON"], handler = "Hide" },
            confirm = { label = L["DIALOG_TEXT_CONFIRM"], handler="DoImport", enabled=false }
        }, importString)

    dialog:Show()
    dialog:Raise()
end