local AddonName, Addon = ...
local locale = Addon:GetLocale()
local UI = Addon.CommonUI.UI
local ImportList = {}



--[[ Validate the list payload ]]
function ImportList:Validate(payload)
    if (type(payload.Items) ~= "table") then

        return false
    end

    local list = payload.Items[1]
    if (type(list.Name) ~= "string") then

        return false
    end

    if (type(list.Description) ~= "string") then

        return false
    end

    if (type(list.Items) ~= "table") then

        return false
    end

    return true
end

--[[ Ensure the imported list has a unique name ]]
function ImportList:CreateListName(imported)
    local canidate = imported
    local unique = false
    local loop = 1

    while (not unique) do
        unique = true
        local name = string.lower(canidate)
        for _, list in ipairs(Addon:GetLists()) do
            if (name == string.lower(list:GetName())) then
                unique = false
                break
            end
        end

        if (not unique) then
            if (loop == 1) then
                canidate = locale:FormatString("IMPORTLIST_UNIQUE_NAME0", imported)
            else
                canidate = locale:FormatString("IMPORTLIST_UNIQUE_NAME1", imported, loop)
            end
            loop = loop + 1
        end
    end

    return canidate
end

--[[ Create the UI to show what you are importing ]]
function ImportList:CreateUI(parent, payload)
    local list = payload.Items[1]
    list.ImportName = self:CreateListName(list.Name)
    
    local items = 0
    for id, _ in pairs(list.Items) do
        items = items + 1
    end

    -- 1 is the name of the list
    -- 2 is the descrpiton
    -- 3 & 4 are plauyer/relam
    -- 5 - 6 are items and plural 

    local pluaral = "s"
    if (items == 1) then
        pluaral = ""
    end

    local markdown = locale:FormatString("IMPORTLIST_MARKDOWN_FMT", 
            list.ImportName, list.Description, 
            payload.Player, payload.Realm, items, pluaral)

    return Addon.CommonUI.MarkdownView.Create(parent, markdown)
end

--[[ Handle the importing of the actual list ]]
function ImportList:Import(payload)
    local list = payload.Items[1]


    local newList = Addon:CreateList(list.ImportName, list.Description)
    for id, v in pairs(list.Items) do
        if (v) then
            newList:Add(id)
        end
    end
end

Addon.Features.Import.ImportList = ImportList